# -*- coding: utf-8 -*-
from PyQt4.QtCore import Qt, QPoint, QAbstractTableModel, QVariant
from PyQt4.QtGui import QMessageBox, QCursor, QPixmap

from qgis.core import QGis, QgsRectangle, QgsFeature, QgsPoint
from qgis.gui import QgsMapTool, QgsRubberBand 

from dialogos.identificar import *
from dialogos.utilidades import Utilidades

class IdentificarMapTool( QgsMapTool ):
    """
        Herramienta Identificar, se encarga de mostrar informacin de los
        elementos espaciales del mapa.
    """
    def __init__( self, parent, canvas ):
        QgsMapTool.__init__( self, canvas )
        self.parent = parent
        self.canvas = canvas

        self.rubberBand = QgsRubberBand( self.canvas, False )
        self.dlgIdentificarVector = DialogoIdentificarVector.obtenerInstancia( self.parent )
        self.dlgIdentificarRaster = DialogoIdentificarRaster.obtenerInstancia( self.parent )
        self.utils = Utilidades.obtenerInstancia()
        self.activeLayer = None

        self.cursor = QCursor( QPixmap( [ "16 16 3 1",
                        "# c None","a c #000000",". c #ffffff",
                        ".###########..##",
                        "...########.aa.#",
                        ".aa..######.aa.#",
                        "#.aaa..#####..##",
                        "#.aaaaa..##.aa.#",
                        "##.aaaaaa...aa.#",
                        "##.aaaaaa...aa.#",
                        "##.aaaaa.##.aa.#",
                        "###.aaaaa.#.aa.#",
                        "###.aa.aaa..aa.#",
                        "####..#..aa.aa.#",
                        "####.####.aa...#",
                        "##########.aa..#",
                        "###########.aa..",
                        "############.a.#",
                        "#############.##" ] ), 1, 1 )

        self.connect( self.dlgIdentificarVector, SIGNAL( "nuevaFeatureSeleccionada" ),
            self.mostrarAtributosFeature )

    def canvasPressEvent( self, event ):
        """ Evento MousePress del canvas """
        # Obetener capa activa
        self.activeLayer = self.parent.activeLayer()        

        if not self.activeLayer:
            self.rubberBand.reset( False )
            self.dlgIdentificarVector.cerrar()
            self.utils.mostrarMensaje( self.parent, "Identificar",
                "Debes seleccionar una capa en la leyenda.", QMessageBox.Warning )
            return

        self.activeLayer.layerId = self.activeLayer.layer().getLayerID()
        p = QPoint( event.pos().x(), event.pos().y() )

        if self.activeLayer.layer().type() == 0: # 0: Vector
            self.dlgIdentificarRaster.cerrar()

            geom = self.activeLayer.layer().dataProvider().geometryType()
            if geom == 1 or geom == 4 or geom == 8 or geom == 11: # Point
                tolerancia = 5
            elif geom == 2 or geom == 5 or geom == 9 or geom == 12: # Polyline
                tolerancia = 3
            elif geom == 3 or geom == 6 or geom == 10 or geom == 13: # Polygon
                tolerancia = 2
            else:
                geom = self.activeLayer.layer().geometryType() # QGis Geometry
                if geom == 0: # Point 
                    tolerancia = 5
                elif geom == 1: # Line
                    tolerancia = 3
                elif geom == 2: # Polygon
                    tolerancia = 2
                else:
                    raise RuntimeError, 'Geometra desconocida: ' + str( geom )

            # Armar una caja alrededor del punto picado
            pmin = self.toMapCoordinates( QPoint( p.x() - tolerancia,
                p.y() - tolerancia ) )
            pmax = self.toMapCoordinates( QPoint(p.x() + tolerancia,
                p.y() + tolerancia ) )
            bBox = QgsRectangle( pmin.x(), pmin.y(), pmax.x(), pmax.y() )

            self.titulos = [ 'CAMPO', 'VALOR' ] # Ttulos para el modelo de atributos
            self.dlgIdentificarVector.actualizarCoordenadasPunto( self.toMapCoordinates( p ),
                self.canvas.mapUnits() )
            self.identificarVector( bBox ) # Llamado a la funcin principal de la clase

        else: # 1: Rster
            self.rubberBand.reset( False )
            self.dlgIdentificarVector.cerrar()
            
            if self.activeLayer.layer().providerKey() == 'wms': # Capa WMS
                self.dlgIdentificarRaster.cerrar()

                # if WMS layer does not cover the view origin,we need to map 
                # the view pixel coordinates to WMS layer pixel coordinates
                viewExtent = self.canvas.extent()
                mapUnitsPerPixel = self.canvas.mapUnitsPerPixel()
                if mapUnitsPerPixel == 0:
                    return
                
                xMinView = viewExtent.xMinimum()
                yMaxView = viewExtent.yMaximum()

                layerExtent = self.activeLayer.layer().extent()
                xMinLayer = layerExtent.xMinimum()
                yMaxLayer = layerExtent.yMaximum()

                if xMinView < xMinLayer:		
                    i = p.x() - ( xMinLayer - xMinView ) / mapUnitsPerPixel
                else:		
                    i = p.x()		

                if yMaxView > yMaxLayer:		
                    j =  p.y() - ( yMaxView - yMaxLayer ) / mapUnitsPerPixel 
                else:
                    j = p.y()

                resp = self.activeLayer.layer().identifyAsText( QgsPoint( i, j ) )

                if resp.isEmpty():		
                    print "*** WMS: RESPUESTA VACIA ***"
                    return

                self.dlgIdentificarWMS = DialogoIdentificarWMS( self.parent )
                self.dlgIdentificarWMS.setMessage( resp )                
                self.dlgIdentificarWMS.mostrar()                
                
            else:
                self.titulos = [ 'BANDA', 'VALOR' ] # Ttulos para el modelo de atributos
                self.dlgIdentificarRaster.actualizarCoordenadasPunto( self.toMapCoordinates( p ),
                    self.canvas.mapUnits() )
                self.identificarRaster( self.toMapCoordinates( event.pos()) )


    def activate( self ):
        self.canvas.setCursor( self.cursor )

    def deactivate( self ):
        """ Borra el rubberBand y cierra el dilogo """
        self.rubberBand.reset( False )
        self.dlgIdentificarVector.cerrar()
        self.dlgIdentificarRaster.cerrar()

    def isZoomTool( self ):
        return False

    def identificarRaster( self, punto ):
        """ Obtiene la informacin del pixel asociado al punto picado """
        mapAttr = self.activeLayer.layer().identify( punto )
        model = ModeloTablas( mapAttr[ 1 ].items(), self.titulos, self ) # Modelo-Vista
        self.dlgIdentificarRaster.tblViewIdentificar.setModel( model )
        self.dlgIdentificarRaster.mostrar()

    def identificarVector( self, bBox ):
        """ Obtiene la informacin del objeto espacial seleccionado """
        self.dlgIdentificarVector.limpiarFeatures()

        provider = self.activeLayer.layer().dataProvider()
        fields = provider.fields()
        feat = QgsFeature()

        # Obtener todos los atributos, interseccin espacial con precisin
        provider.select( provider.attributeIndexes(), bBox, False, True )

        self.dictAttrs = {}
        featCount = 0

        # Obtiene los atributos de cada feature y los almacena en un diccionario
        while provider.nextFeature( feat ):
            featCount += 1
            self.dlgIdentificarVector.agregarFeature( feat.id() )

            attrs = feat.attributeMap()
            aAttrs = []

            for ( k, attr ) in attrs.iteritems():
                aAttrs.append( [ str( fields[ k ].name() ), attr.toString() ] )

            self.dictAttrs[ str( feat.id() ) ] = aAttrs

        if featCount == 0:
            model = ModeloTablas( [], self.titulos, self ) # Modelo-Vista
            self.dlgIdentificarVector.tblViewIdentificar.setModel( model )
            self.rubberBand.reset( False ) # Borrar RubberBand
        else:
            self.dlgIdentificarVector.seleccionarPrimeraFeature() # Dispara SIGNAL

        self.dlgIdentificarVector.mostrar()

    def mostrarAtributosFeature( self, idFeature ):
        """ SLOT mostrarAtributosFeature
            ----------------------------

            Asigna los atributos de la feature seleccionada al modelo atributos y
            resalta la feature en el mapa.
        """
        if not self.activeLayer:
            return

        feat = QgsFeature()
        if not self.activeLayer.layer().featureAtId( int( idFeature ), feat,
            True, False ):
            return

        model = ModeloTablas( self.dictAttrs[ str( idFeature ) ], self.titulos, self )
        self.dlgIdentificarVector.tblViewIdentificar.setModel( model )
        self.highlightFeature( self.activeLayer.layer(), int( idFeature ) )

    def highlightFeature( self, layer, featureId ):
        """ Resalta la geometra correspondiente a la feature seleccionada """
        if not layer:
            return

        feat = QgsFeature()
        if not layer.featureAtId( featureId, feat, True, False ):
            return

        if not feat.geometry():
            return

        # Inicializar RubberBand
        self.rubberBand.reset( feat.geometry().type() == QGis.Polygon )
        self.rubberBand = QgsRubberBand( self.canvas, feat.geometry().type() == QGis.Polygon )

        if ( self.rubberBand ): # Definir y mostrar RubberBand
            self.rubberBand.setToGeometry( feat.geometry(), layer )
            self.rubberBand.setWidth( 2 )
            self.rubberBand.setColor( Qt.red )
            self.rubberBand.show()

    def capaRemovida( self, layerId ):
        """ SLOT capaRemovida
            -----------------

            Responde al SIGNAL layerWillBeRemoved del QgsMapLayerRegistry.

            Si la capa a remover del canvas coincide con la capa activa del
            identify se limpia el dilogo y se borra el RubberBand. Esto evita
            un fallo de segmentacin cuando el dilogo contiene mltiples
            features de una capa removida.
        """
        if self.activeLayer:
            if layerId == self.activeLayer.layerId:
                self.rubberBand.reset( False )
                self.dlgIdentificarVector.limpiarFeatures()
                model = ModeloTablas( [], ['',''], self )
                self.dlgIdentificarVector.tblViewIdentificar.setModel( model )

class ModeloTablas( QAbstractTableModel ):
    """
        Clase ModeloTablas
        ------------------
        Provee el modelo de almacenamiento para las tablas.
        Implementa el Modelo Vista.
    """
    def __init__( self, arreglo, encabezados, parent=None, *args ):
        QAbstractTableModel.__init__( self, parent, *args )
        self.datos = arreglo
        self.headerdata = encabezados
        self.numFilas = len( self.datos )

    def rowCount( self, parent ):
        """ Retorna el nmero de filas en la tabla """
        return len( self.datos )

    def columnCount( self, parent ):
        """ Retorna el nmero de columnas en la tabla """
        if len( self.datos ) > 0:
            return len( self.datos[ 0 ] )
        else:
            return 0

    def data(self, index, role):
        """ Retorna los datos de la tabla """
        if role == Qt.DisplayRole:
            return QVariant( self.datos[ index.row() ][ index.column() ] )
        elif role ==  Qt.TextAlignmentRole: # Posicin del texto en la celda
            return QVariant( Qt.AlignCenter )

        return QVariant()

    def headerData( self, section, orientation, role ):
        """ Retorna los encabezados de la tabla """
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:            
            return QVariant( self.headerdata[ section ] )
        return QVariant()