"""
Linear Referencing plugin
(c) Copyright 2008 Martin Dobias

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.       
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from dlgaddrouteevents_ui import Ui_DlgAddRouteEvents

from dlgdberror import DlgDbError
import postgis

from qgis.core import QgsDataSourceURI


class DlgAddRouteEvents(QDialog, Ui_DlgAddRouteEvents):

	def __init__(self, db, iface, parent=None):
		QDialog.__init__(self, parent)
		self.db = db
		self.iface = iface
		
		self.setupUi(self)
		
		self.connect(self.buttonBox, SIGNAL("accepted()"), self.onOK)
		
		#self.connect(self.cboRefRoute, SIGNAL("currentIndexChanged(int)"), self.populateRouteIds)
		self.connect(self.cboEventsTable, SIGNAL("currentIndexChanged(int)"), self.populateEventsColumns)
		
		self.connect(self.radEventsPoint, SIGNAL("clicked()"), self.updateUi)
		self.connect(self.radEventsLinear, SIGNAL("clicked()"), self.updateUi)
		
		self.populateRoutes()
		self.populateEventsTables()
		self.updateUi()
		
		# TODO: add a warning if there are no available routes or events tables
		

	
	def updateUi(self):
		""" when checked point/linear event """
		isPoint = self.radEventsPoint.isChecked()
		self.labelMeasure2.setEnabled(not isPoint)
		self.cboEventsMeasure2.setEnabled(not isPoint)
		
	
	def populateRoutes(self):	
		""" populate combo box with available routes """
		routes = self.db.list_routes_2()
		#print routes
		
		self.cboRefRoute.clear()
		for route in sorted(routes):
			self.cboRefRoute.addItem(route)
	
	
	#def populateRouteIds(self):
	#	""" populate combo box with columns of selected route """
	#	table = self.cboRefRoute.currentText()
	#	routeIds = self.db.list_table_columns(table)
	#	print routeIds
	#
	#	self.cboRouteId.clear()
	#	for routeId in routeIds:
	#		self.cboRouteId.addItem(routeId)
			
	
	def populateEventsTables(self):
		""" populate combo box with possible events tables from database """
		tables = self.db.list_aspatial_tables()
		#print tables
		
		self.cboEventsTable.clear()
		for table in sorted(tables):
			self.cboEventsTable.addItem(table)
	
	
	def populateEventsColumns(self):
		""" populate combo box with events table's columns """
		table = self.cboEventsTable.currentText()
		columns = self.db.list_table_columns(table)
		#print columns
		
		self.cboEventsMeasure.clear()
		self.cboEventsMeasure2.clear()
		self.cboEventsRoute.clear()
		for column in columns:
			self.cboEventsMeasure.addItem(column)
			self.cboEventsMeasure2.addItem(column)
			self.cboEventsRoute.addItem(column)
			
		# set suggested name
		self.editOutput.setText(table+"_events")
	
	
	def onOK(self):
		""" do the SKA """
		
		route_table = self.cboRefRoute.currentText()
		route = self.db.get_route_info(route_table)
		events_table = self.cboEventsTable.currentText()
		events_route_id = self.cboEventsRoute.currentText()
		events_measure = self.cboEventsMeasure.currentText()
		output = self.editOutput.text()
		
		if self.db.table_exists(output):
			QMessageBox.critical(self, "error", "The output table exists already!")
			return
		
		is_point = self.radEventsPoint.isChecked()
		if not is_point:
			events_measure2 = self.cboEventsMeasure2.currentText()
		
		if not route:
			QMessageBox.critical(self, "error", "Couldn't find info about route '%s'" % route_table)
		
		# LEGACY CODE
		# find out route's geometry column
		#route_route_id = self.cboRouteId.currentText()
		#route_geom_column = self.db.get_route_geom_column(route_table)
		#route = postgis.Route(route_table, route_route_id, route_geom_column)
		
		QApplication.instance().setOverrideCursor(QCursor(Qt.WaitCursor))
		try:
			if is_point:
				self.db.create_table_with_located_points(route, events_table, events_route_id, events_measure, output)
			else:
				self.db.create_table_with_located_lines(route, events_table, events_route_id, events_measure, events_measure2, output)
		except postgis.DatabaseError, e:
			QApplication.instance().restoreOverrideCursor()
			DlgDbError.showError(e)
			return
		
		QApplication.instance().restoreOverrideCursor()
		
		# load new layer
		uri = QgsDataSourceURI()
		uri.setConnection(self.db.host, str(self.db.port), self.db.dbname, self.db.user, self.db.passwd)
		uri.setDataSource('public', output, 'geom_event')
		self.iface.addVectorLayer(uri.uri(), output, "postgres")
		
		self.accept()
